class MediaFileTrackingEvent():

	def __init__(self, entry_type, description, context_information={}):
		self.__type=entry_type
		self.__description=description
		self.__context_info=context_information

	def type(self):
		return self.__type

	def description(self):
		return self.__description

	def context_information(self):
		return self.__context_info

	def process(self, file_descriptor):
		return file_descriptor

class CompressedAudioFileFound(MediaFileTrackingEvent):
	def __init__(self, tracking_id):
		MediaFileTrackingEvent.__init__(self, "COMPRESSED_AUDIOFILE_FOUND", "Related compressed audio file found")
		self.__tracking_id=tracking_id

	def process(self, file_descriptor):
		file_descriptor.audiofiles.append(self.__tracking_id)
		return file_descriptor

class FileCouldNotBeInflated(MediaFileTrackingEvent):		
	def __init__(self):
		MediaFileTrackingEvent.__init__(self, "INFLATE_PROCESS_FAILURE", "File could not be inflated")

	def process(self, file_descriptor):
		file_descriptor.inflated=False
		return file_descriptor	


from longtang.actors import messages

class LookupCoverArt(messages.ActorMessage):

	def __init__(self, artist, album, target_dir_path):
		messages.ActorMessage.__init__(self)
		self.__artist=artist
		self.__album=album
		self.__target_dir_path=target_dir_path

	def artist(self):
		return self.__artist

	def album(self):
		return self.__album

	def target_dir_path(self):
		return self.__target_dir_path

class CoverArtFound(messages.ActorMessage):

	def __init__(self, artist, album, cover_path):
		messages.ActorMessage.__init__(self)
		self.__artist=artist
		self.__album=album
		self.__cover_path=cover_path

	def artist(self):
		return self.__artist

	def album(self):
		return self.__album

	def cover_path(self):
		return self.__cover_path

class CoverArtRetrievalFailed(messages.ActorMessage):

	def __init__(self, artist, album):
		messages.ActorMessage.__init__(self)
		self.__artist=artist
		self.__album=album

	def artist(self):
		self.__artist

	def album(self):
		self.__album



# -*- coding: utf-8 -*-

#http://docs.python.org/2/library/unittest.html#unittest.TestCase.assertEqual

import unittest
import os

from hamcrest import *
from longtang.actors.inbound import filepoller, messages as fp_messages
from longtang.actors import actorstestutils, messages
from longtang.system.properties import VerbosityLevel

class TestFilePollerActor(unittest.TestCase):

    def test_polling(self):

    	data_dir = os.path.join(actorstestutils.current_dirpath(__file__), 'data')

        poller_tester = actorstestutils.TestActorBuilder().with_type(filepoller.FilePoller)\
                                                          .verbosity(VerbosityLevel.DEBUG)\
                                                          .build()

        poller_tester.tell(fp_messages.StartPolling(data_dir))

        assert_that(poller_tester.inspector().num_instances(), is_(equal_to(5)), 'Total amount of messages received is wrong')
        assert_that(poller_tester.inspector().num_instances(fp_messages.AudioFileFound), is_(equal_to(3)), 'Total amount of AudioFileFound messages received is wrong')
        assert_that(poller_tester.inspector().num_instances(fp_messages.FilePollingDone), is_(equal_to(1)), 'Total amount of FilePollingDone messages received is wrong')
        assert_that(poller_tester.inspector().num_instances(messages.Terminated), is_(equal_to(1)), 'Total amount of Terminated messages received is wrong')

        assert_that(poller_tester.inspector().first(fp_messages.AudioFileFound).filepath(), is_(not_none()),'Filepath attribute is empty or wrong')

    def test_weird_encoded_source_polling(self):

        data_dir = os.path.join(actorstestutils.current_dirpath(__file__), 'anóther_daña')

        poller_tester = actorstestutils.TestActorBuilder().with_type(filepoller.FilePoller)\
                                                          .verbosity(VerbosityLevel.DEBUG)\
                                                          .build()

        poller_tester.tell(fp_messages.StartPolling(data_dir))

        assert_that(poller_tester.inspector().num_instances(), is_(equal_to(4)), 'Total amount of messages received is wrong')
        assert_that(poller_tester.inspector().num_instances(fp_messages.AudioFileFound), is_(equal_to(2)), 'Total amount of AudioFileFound messages received is wrong')
        assert_that(poller_tester.inspector().num_instances(fp_messages.FilePollingDone), is_(equal_to(1)), 'Total amount of FilePollingDone messages received is wrong')
        assert_that(poller_tester.inspector().num_instances(messages.Terminated), is_(equal_to(1)), 'Total amount of Terminated messages received is wrong')

        assert_that(poller_tester.inspector().first(fp_messages.AudioFileFound).filepath(), is_(not_none()),'Filepath attribute is empty or wrong')

    def test_compressed_file_polling(self):

        data_dir = os.path.join(actorstestutils.current_dirpath(__file__), 'compressed_data')

        poller_tester = actorstestutils.TestActorBuilder().with_type(filepoller.FilePoller)\
                                                          .verbosity(VerbosityLevel.DEBUG)\
                                                          .termination_type(fp_messages.FilePollingDone)\
                                                          .build()

        poller_tester.tell(fp_messages.StartPolling(data_dir))

        assert_that(poller_tester.inspector().num_instances(), is_(equal_to(11)), 'Total amount of messages received is wrong')
        assert_that(poller_tester.inspector().num_instances(fp_messages.CompressedAudioFileFound), is_(equal_to(9)), 'Total amount of CompressedAudioFileFound messages received is wrong')
        assert_that(poller_tester.inspector().num_instances(fp_messages.FilePollingDone), is_(equal_to(1)), 'Total amount of FilePollingDone messages received is wrong')
        assert_that(poller_tester.inspector().num_instances(messages.Terminated), is_(equal_to(1)), 'Total amount of Terminated messages received is wrong')

        assert_that(poller_tester.inspector().first(fp_messages.CompressedAudioFileFound).filepath(), is_(not_none()), 'File path attribute is empty or wrong')
        assert_that(poller_tester.inspector().first(fp_messages.CompressedAudioFileFound).compressed_parent_filepath(), is_(not_none()), 'Compressed parent filepath attribute is empty or wrong')

    def test_compressed_file_polling_failure(self):

        data_dir = os.path.join(actorstestutils.current_dirpath(__file__), 'compressed_data_failure')

        poller_tester = actorstestutils.TestActorBuilder().with_type(filepoller.FilePoller)\
                                                          .verbosity(VerbosityLevel.DEBUG)\
                                                          .termination_type(fp_messages.FilePollingDone)\
                                                          .build()

        poller_tester.tell(fp_messages.StartPolling(data_dir))

        assert_that(poller_tester.inspector().num_instances(), is_(equal_to(5)), 'Total amount of messages received is wrong')
        assert_that(poller_tester.inspector().num_instances(fp_messages.CompressedFileCouldNotBeOpened), is_(equal_to(3)), 'Total amount of CompressedFileCouldNotBeOpened messages received is wrong')
        assert_that(poller_tester.inspector().num_instances(fp_messages.FilePollingDone), is_(equal_to(1)), 'Total amount of FilePollingDone messages received is wrong')
        assert_that(poller_tester.inspector().num_instances(messages.Terminated), is_(equal_to(1)), 'Total amount of Terminated messages received is wrong')

        assert_that(poller_tester.inspector().first(fp_messages.CompressedFileCouldNotBeOpened).filepath(), is_(not_none()), 'File path attribute is empty or wrong from message type CompressedFileCouldNotBeOpened ')

    def test_mix_content_polling(self):
        data_dir = os.path.join(actorstestutils.current_dirpath(__file__), 'mixed_data')

        poller_tester = actorstestutils.TestActorBuilder().with_type(filepoller.FilePoller)\
                                                          .verbosity(VerbosityLevel.DEBUG)\
                                                          .termination_type(fp_messages.FilePollingDone)\
                                                          .build()

        poller_tester.tell(fp_messages.StartPolling(data_dir))

        assert_that(poller_tester.inspector().num_instances(), is_(equal_to(8)), 'Total amount of messages received is wrong')
        assert_that(poller_tester.inspector().num_instances(fp_messages.CompressedFileCouldNotBeOpened), is_(equal_to(1)), 'Total amount of CompressedFileCouldNotBeOpened messages received is wrong')
        assert_that(poller_tester.inspector().num_instances(fp_messages.CompressedAudioFileFound), is_(equal_to(3)), 'Total amount of CompressedAudioFileFound messages received is wrong')
        assert_that(poller_tester.inspector().num_instances(fp_messages.AudioFileFound), is_(equal_to(2)), 'Total amount of AudioFileFound messages received is wrong')
        assert_that(poller_tester.inspector().num_instances(messages.Terminated), is_(equal_to(1)), 'Total amount of Terminated messages received is wrong')

if __name__ == '__main__':
    unittest.main()    


class MediaFileTracking():
	def __init__(self, descriptor):
		self.__descriptor=descriptor
		self.__events_container=TrackingEventsProcessor([])

	@staticmethod
	def create_from(descriptor):
		return MediaFileTracking(descriptor)

	@property
	def mediafile(self):
		return self.__descriptor

	@property
	def tracking_events(self):
		return self.__events_container.events_registry()

	def track_activity(self, track_event):
		self.__descriptor = self.__events_container.process(self.__descriptor, track_event)

class FileDescriptor:
	def __init__(self):
		self.__sourcepath=None
		self.__destinationpath=None

	@property
	def sourcepath(self):
		return self.__sourcepath

	@sourcepath.setter
	def sourcepath(self, value):
		self.__sourcepath=value

class AudioFileDescriptor(FileDescriptor):
	def __init__(self):
		FileDescriptor.__init__(self)
		self.__destinationpath=None

	@property
	def destinationpath(self):
		return self.__destinationpath

	@destinationpath.setter
	def destinationpath(self, value):
		self.__destinationpath=value

class CompressedAudioFileDescriptor(AudioFileDescriptor):
	def __init__(self):
		AudioFileDescriptor.__init__(self)
		self.__compressed_file=None

	@property
	def compressed_file(self):
		return self.__compressed_file

class CompressedFileDescriptor(FileDescriptor):
	def __init__(self):
		FileDescriptor.__init__(self)
		self.__audiofiles=[]
		self.__inflated=True

	@property
	def audiofiles(self):

		class AudioFilesBrowser:
			def __init__(self, audiofiles):
				self.__audiofiles = audiofiles

			def size(self):
				return len(self.__audiofiles)

			def append(self, related_tracking_id):
				self.__audiofiles.append(related_tracking_id)

			def all(self):
				return self.__audiofiles

		return AudioFilesBrowser(self.__audiofiles)

	@property
	def inflated(self):
		return self.__inflated

	@inflated.setter
	def inflated(self, value):
		self.__inflated=value

class AudioFileDescriptorBuilder:
	def __init__(self):
		self.__sourcepath=None

	def sourcepath(self, sourcepath):
		self.__sourcepath=sourcepath
		return self

	def build(self):
		descriptor=AudioFileDescriptor()
		descriptor.sourcepath=self.__sourcepath

		return descriptor

class CompressedAudioFileDescriptorBuilder:
	def __init__(self):
		self.__compressed_file_id=None
		self.__sourcepath=None

	def sourcepath(self, sourcepath):
		self.__sourcepath=sourcepath
		return self

	def compressed_file(self, compressed_file_tracking_id):
		self.__compressed_file_id=compressed_file_tracking_id
		return self

	def build(self):
		descriptor=CompressedAudioFileDescriptor()
		descriptor.sourcepath=self.__sourcepath
		descriptor.compressed_file=self.__compressed_file_id

		return descriptor		

class CompressedFileDescriptorBuilder:
	def __init__(self):
		self.__sourcepath=None

	def sourcepath(self, sourcepath):
		self.__sourcepath=sourcepath
		return self

	def build(self):
		descriptor=CompressedFileDescriptor()
		descriptor.sourcepath=self.__sourcepath

		return descriptor

class TrackingEventsBrowser:
	def __init__(self, entries):
		self.__tracking_groups={}
		self.__tracking_entries=entries

		self.__group_entries(entries)

	def __group_entries(self, entries):
		for entry in entries:
			self.__assign_event_to_group(entry)

	def __assign_event_to_group(self, tracking_entry):
		if self.__tracking_groups.has_key(tracking_entry.type()):
			self.__tracking_groups[tracking_entry.type()].append(tracking_entry)
		else:
			self.__tracking_groups[tracking_entry.type()]=[tracking_entry]			

	def size(self):
		return len(self.__tracking_entries)

	def is_empty(self):
		return self.size() == 0

	def all(self):
		return TrackingEventsBrowser(self.__tracking_entries)

	def get(self, index):
		return self.__tracking_entries[index]

	def of_type(self, event_type):
		if self.__tracking_groups.has_key(event_type):
			return TrackingEventsBrowser(self.__tracking_groups[event_type])
		else:
			return TrackingEventsBrowser([])

	def __iter__(self):
		return iter(self.__tracking_entries)

class TrackingEventsProcessor():
	def __init__(self, entries):
		self.__tracking_entries=entries

	def process(self, current_descriptor, tracking_entry):
		self.__tracking_entries.append(tracking_entry)
		return tracking_entry.process(current_descriptor)

	def events_registry(self):
		return TrackingEventsBrowser(self.__tracking_entries)



#https://stuffivelearned.org/doku.php?id=programming:python:albumcoverdl

import messages
import os.path
import urllib2

from lxml import etree

from amazonproduct import API
from longtang.actors import actors
from longtang.common import domain as common_domain

#http://aws-portal.amazon.com/gp/aws/developer/account/index.html
#http://python-amazon-product-api.readthedocs.org/en/latest/pagination.html

class AlbumCoverArtActor(actors.Actor):

    __api=API('AKIAIUHQBFQK7T2VSSTQ','KLoVSu3WB3n3Nj4pCjsIue1SOSE14ynO4WjffFCc','us')
    __SEARCH_GROUP='Music'
    __RESPONSE_GROUP='Images'

    def receive(self, message):
        if isinstance(message, messages.LookupCoverArt):
            self.logger().debug(u'Downloading album art from "{0} - {1}" .....'.format(message.artist(), message.album()), None)

            result=None

            try:
                result = self.__api.item_search('Music', ResponseGroup='Images', Keywords='{0} {1}'.format(message.artist() , message.album()), AssociateTag='ws')
            except Exception , e:
                # try just the album
                try:
                    result = self.__api.item_search('Music', ResponseGroup='Images', Keywords='{0}'.format(message.album()), AssociateTag='ws')
                except Exception, e:
                    self.logger().error(u'Album art search from "{0} - {1}" failed: {2}'.format(message.artist(), message.album(), e), None)                    

            if result is not None:

                target_local_file=os.path.join(message.target_dir_path(), 'cover.jpg')

                block_sz = 8192

                for item in result:

                    try:
                        url=str(item.ImageSets.ImageSet.LargeImage.URL)
                    except:
                        url=str(item.ImageSets.ImageSet.MediumImage.URL)

                    #print(etree.tostring(item, pretty_print=True))

                    with open(target_local_file, 'wb') as target:

                        try:
                            fh = urllib2.urlopen(url)

                            while True:
                                buffer = fh.read(block_sz)
                                
                                if not buffer:
                                    break

                                target.write(buffer)

                            fh.close()
                            
                            self.sender().tell(messages.CoverArtFound(message.artist(), message.album(), target_local_file), self.myself())

                            self.logger().info(u'Album art from "{0} - {1}" successfully downloaded.'.format(message.artist(), message.album()), None)

                        except urllib2.URLError, e:
                            self.logger().error(u'Album art retrieval from "{0} - {1}" failed: {2}'.format(message.artist(), message.album(), e), None)
                            self.sender().tell(messages.CoverArtRetrievalFailed(message.artist(), message.album()), self.myself())

                    break
            else:
                self.logger().error(u'Album art from "{0} - {1}" was not found.'.format(message.artist(), message.album()), None)
                self.sender().tell(messages.CoverArtRetrievalFailed(message.artist(), message.album()), self.myself())


import os
import tempfile

from longtang.actors.inbound.inflate.strategies.registry import available_formats

class PumpingClerkFactory:

	@staticmethod
	def from_file(filepath):
		filename, fileext_original = os.path.splitext(filepath.lower())

		fileext = fileext_original[1:]

		if fileext in available_formats:
			return PumpingClerk(filepath, available_formats[fileext])
		else:
			class PumpingClerkNotAvailable(Exception):
				pass
			
			raise PumpingClerkNotAvailable('Pumping clerk for type \'{0}\' not found'.format(fileext))

class PumpingClerk:
	def __init__(self, filepath, pump_strategy):
		self.__filepath=filepath
		self.__strategy=pump_strategy
		self.__inflate_details=None

	def inflate(self):
		file_ref = self.__strategy.open(self.__filepath)
		self.__inflate_details = self.__strategy.inflate_at(file_ref, tempfile.mkdtemp(prefix=u'pump_'))
		file_ref.close()

		return self

	def entries(self):
		return self.__inflate_details.entries()


import messages
import summary
import tracking_generator as tg

from longtang.actors import actors
from longtang.common.mediatracking import mediatracking, events as media_events

class TrackingActor(actors.Actor):

	def __init__(self, system_ref, unique_id):
		actors.Actor.__init__(self, system_ref, unique_id)

		self.__audio_trackings={}
		self.__compressed_files_trackings={}

	def receive(self, message):
		if isinstance(message, messages.CreateCompressedFileTrackingEntry):

			self.logger().info(u'Creating compressed file tracking entry for source path \'{0}\' .....'.format(message.sourcepath()))

			descriptor = mediatracking.CompressedFileDescriptorBuilder().sourcepath(message.sourcepath()).build()
			file_tracking = mediatracking.MediaFileTracking.create_from(descriptor)
			trackingid = tg.MessageTrackingGenerator.generate(message)

			self.__compressed_files_trackings[trackingid]=file_tracking

			self.sender().tell(messages.CompressedFileTrackingEntryCreated(trackingid), self.myself())

		elif isinstance(message, messages.CreateCompressedAudioFileTrackingEntry):
			
			self.logger().info(u'Creating compressed audio file tracking entry [path={0}, parent={1}] .....'.format(message.sourcepath(), message.parent_sourcepath()))

			compressedfile_tracking = None
			compressedfile_trackingid = tg.MessageTrackingGenerator.generate_parent(message)

			if compressedfile_trackingid not in self.__compressed_files_trackings:
				descriptor = mediatracking.CompressedFileDescriptorBuilder().sourcepath(message.parent_sourcepath()).build()
				compressedfile_tracking = mediatracking.MediaFileTracking.create_from(descriptor)
				self.__compressed_files_trackings[compressedfile_trackingid]=compressedfile_tracking				
			else:
				compressedfile_tracking = self.__compressed_files_trackings[compressedfile_trackingid]

			audiofile_descriptor = mediatracking.CompressedAudioFileDescriptorBuilder().sourcepath(message.sourcepath())\
																			 			.compressed_file(compressedfile_trackingid)\
																			 		    .build()
		
			audiofile_tracking = mediatracking.MediaFileTracking.create_from(audiofile_descriptor)
			audiofile_trackingid = tg.MessageTrackingGenerator.generate(message)

			#Register parent's audio file
			compressedfile_tracking.track_activity(media_events.CompressedAudioFileFound(audiofile_trackingid))
			#Register compressed audio file
			self.__audio_trackings[audiofile_trackingid]=audiofile_tracking

			self.logger().debug(u'Compressed audio file tracked under id {0}.....'.format(audiofile_trackingid))

			self.sender().tell(messages.CompressedAudioFileTrackingEntryCreated(audiofile_trackingid, compressedfile_trackingid), self.myself())

		elif isinstance(message, messages.CreateTrackingEntry):
			self.logger().info(u'Creating tracking entry for source path {0} .....'.format(message.sourcepath()))

			descriptor = mediatracking.AudioFileDescriptorBuilder().sourcepath(message.sourcepath()).build()
			file_tracking = mediatracking.MediaFileTracking.create_from(descriptor)

			trackingid = tg.MessageTrackingGenerator.generate(message)

			self.__audio_trackings[trackingid]=file_tracking

			self.sender().tell(messages.TrackingEntryCreated(trackingid), self.myself())

		elif isinstance(message, messages.LookupTrackingEntry):
			self.logger().info(u'Looking up tracking entry {0} .....'.format(message.tracking_id()))
			
			if self.__audio_trackings.has_key(message.tracking_id()):
				self.sender().tell(messages.TrackingEntryFound(self.__audio_trackings[message.tracking_id()]), self.myself())
			elif message.tracking_id() in self.__compressed_files_trackings:
				self.sender().tell(messages.TrackingEntryFound(self.__compressed_files_trackings[message.tracking_id()]), self.myself())
			else:
				self.sender().tell(messages.TrackingEntryNotFound(message.tracking_id()), self.myself())	

		elif isinstance(message, messages.RegisterTrackingEvent):
			self.logger().info(u'Event of type \'{1}\' recieved for tracking id \'{0}\' .....'.format(message.tracking_id(),message.tracking_event().type()))

			trackings = self.__audio_trackings 

			if isinstance(message, messages.RegisterCompressedFileTrackingEvent):
				trackings = self.__compressed_files_trackings

			if trackings.has_key(message.tracking_id()):
				trackings[message.tracking_id()].track_activity(message.tracking_event())

				if self.sender() is not None:
					self.sender().tell(messages.TrackingEventSuccessfullyRegistered(), self.myself())
			else:

				if self.sender() is not None:
					self.sender().tell(messages.TrackingEntryNotFound(message.tracking_id()), self.myself())

		elif isinstance(message, messages.GenerateSummary):
			self.sender().tell(messages.SummarySuccessfullyGenerated(summary.ActivitySummary.create(self.__audio_trackings, self.__compressed_files_trackings)), self.myself())			
		else: 
			self.notify_marooned_message(message)


# -*- coding: utf-8 -*-

import unittest
import os
from hamcrest import *

from longtang.actors import actorstestutils, messages
from longtang.actors.inbound.inflate import filepump, messages as fp_messages
from longtang.system.properties import VerbosityLevel

class TestFilePump(unittest.TestCase):
	def test_zip_pump(self):

		test_zip_file = os.path.join(actorstestutils.current_dirpath(__file__), 'data', 'data.zip')

		pump_tester = actorstestutils.TestActorBuilder().with_type(filepump.FilePumpActor)\
														.termination_type(fp_messages.InflateFileDone)\
														.verbosity(VerbosityLevel.DEBUG)\
														.termination_moratorium(2)\
														.build()

		pump_tester.tell(fp_messages.InflateFile(test_zip_file))

		assert_that(pump_tester.inspector().num_instances(fp_messages.InflatedFileAvailable), is_(equal_to(3)), 'Total amount of InflatedFileAvailable messages received is wrong')
		assert_that(pump_tester.inspector().num_instances(fp_messages.InflateFileDone), is_(equal_to(1)), 'Total amount of PumpFileDone messages received is wrong')

		for message in pump_tester.inspector().all(fp_messages.InflatedFileAvailable):
			assert_that(message.filepath(), is_(not_none()), 'Filepath attribute is empty')

			assert_that(message.filepath(), is_(any_of(contains_string('dummy1.mp3'),\
													   contains_string('dummy2.mp3'),\
													   contains_string('dummy3.mp3'))), 'Filepath attribute might be wrong')

			assert_that(message.parent_filepath(), is_(not_none()), 'Parent inflated filepath attribute is empty')
			assert_that(message.parent_filepath(), is_(equal_to(test_zip_file)), 'Parent inflated filepath attribute is wrong')

		assert_that(pump_tester.inspector().first(fp_messages.InflateFileDone).filepath(), is_(not_none()), 'InflateFileDone message attribute ''filepath'' is empty')
		assert_that(pump_tester.inspector().first(fp_messages.InflateFileDone).filepath(), is_(equal_to(test_zip_file)), 'InflateFileDone message attribute ''filepath'' is wrong')

	def test_zip_pump_failure(self):
		test_zip_file = os.path.join(actorstestutils.current_dirpath(__file__), 'data', 'data_failure.zip')

		pump_tester = actorstestutils.TestActorBuilder().with_type(filepump.FilePumpActor)\
														.termination_type(fp_messages.InflateFileFailed)\
														.verbosity(VerbosityLevel.DEBUG)\
														.termination_moratorium(2)\
														.build()

		pump_tester.tell(fp_messages.InflateFile(test_zip_file))

		assert_that(pump_tester.inspector().num_instances(fp_messages.InflateFileFailed), is_(equal_to(1)), 'Total amount of InflatedFileFailed messages received is wrong')
		assert_that(pump_tester.inspector().first(fp_messages.InflateFileFailed).filepath(), is_(not_none()), 'InflatedFileFailed ''filepath'' is empty')
		assert_that(pump_tester.inspector().first(fp_messages.InflateFileFailed).filepath(), is_(equal_to(test_zip_file)), 'InflatedFileFailed ''filepath'' is wrong')

	def test_rar_pump(self):
		test_rar_file = os.path.join(actorstestutils.current_dirpath(__file__), 'data', 'data.rar')

		pump_tester = actorstestutils.TestActorBuilder().with_type(filepump.FilePumpActor)\
														.termination_type(fp_messages.InflateFileDone)\
														.verbosity(VerbosityLevel.DEBUG)\
														.termination_moratorium(2)\
														.build()

		pump_tester.tell(fp_messages.InflateFile(test_rar_file))

		assert_that(pump_tester.inspector().num_instances(fp_messages.InflatedFileAvailable), is_(equal_to(3)), 'Total amount of InflatedFileAvailable messages received is wrong')
		assert_that(pump_tester.inspector().num_instances(fp_messages.InflateFileDone), is_(equal_to(1)), 'Total amount of PumpFileDone messages received is wrong')

		for message in pump_tester.inspector().all(fp_messages.InflatedFileAvailable):
			assert_that(message.filepath(), is_(not_none()), 'Filepath attribute is empty')

			assert_that(message.filepath(), is_(any_of(contains_string('dummy1.mp3'),\
													   contains_string('dummy2.mp3'),\
													   contains_string('dummy3.mp3'))), 'Filepath attribute might be wrong')

			assert_that(message.parent_filepath(), is_(not_none()), 'Parent inflated filepath attribute is empty')
			assert_that(message.parent_filepath(), is_(equal_to(test_rar_file)), 'Parent inflated filepath attribute is wrong')

		assert_that(pump_tester.inspector().first(fp_messages.InflateFileDone).filepath(), is_(not_none()), 'InflateFileDone message attribute ''filepath'' is empty')
		assert_that(pump_tester.inspector().first(fp_messages.InflateFileDone).filepath(), is_(equal_to(test_rar_file)), 'InflateFileDone message attribute ''filepath'' is wrong')

	def test_rar_pump_failure(self):
		test_rar_file = os.path.join(actorstestutils.current_dirpath(__file__), 'data', 'data_failure.rar')

		pump_tester = actorstestutils.TestActorBuilder().with_type(filepump.FilePumpActor)\
														.termination_type(fp_messages.InflateFileFailed)\
														.verbosity(VerbosityLevel.DEBUG)\
														.termination_moratorium(2)\
														.build()

		pump_tester.tell(fp_messages.InflateFile(test_rar_file))

		assert_that(pump_tester.inspector().num_instances(fp_messages.InflateFileFailed), is_(equal_to(1)), 'Total amount of InflatedFileFailed messages received is wrong')
		assert_that(pump_tester.inspector().first(fp_messages.InflateFileFailed).filepath(), is_(not_none()), 'InflatedFileFailed ''filepath'' is empty')
		assert_that(pump_tester.inspector().first(fp_messages.InflateFileFailed).filepath(), is_(equal_to(test_rar_file)), 'InflatedFileFailed ''filepath'' is wrong')

	def test_7z_pump(self):
		test_7zip_file = os.path.join(actorstestutils.current_dirpath(__file__), 'data', 'data.7z')

		pump_tester = actorstestutils.TestActorBuilder().with_type(filepump.FilePumpActor)\
														.termination_type(fp_messages.InflateFileDone)\
														.verbosity(VerbosityLevel.DEBUG)\
														.termination_moratorium(2)\
														.build()

		pump_tester.tell(fp_messages.InflateFile(test_7zip_file))

		assert_that(pump_tester.inspector().num_instances(fp_messages.InflatedFileAvailable), is_(equal_to(3)), 'Total amount of InflatedFileAvailable messages received is wrong')
		assert_that(pump_tester.inspector().num_instances(fp_messages.InflateFileDone), is_(equal_to(1)), 'Total amount of PumpFileDone messages received is wrong')

		for message in pump_tester.inspector().all(fp_messages.InflatedFileAvailable):
			assert_that(message.filepath(), is_(not_none()), 'Filepath attribute is empty')

			assert_that(message.filepath(), is_(any_of(contains_string('dummy1.mp3'),\
													   contains_string('dummy2.mp3'),\
													   contains_string('dummy3.mp3'))), 'Filepath attribute might be wrong')

			assert_that(message.parent_filepath(), is_(not_none()), 'Parent inflated filepath attribute is empty')
			assert_that(message.parent_filepath(), is_(equal_to(test_7zip_file)), 'Parent inflated filepath attribute is wrong')

		assert_that(pump_tester.inspector().first(fp_messages.InflateFileDone).filepath(), is_(not_none()), 'InflateFileDone message attribute ''filepath'' is empty')
		assert_that(pump_tester.inspector().first(fp_messages.InflateFileDone).filepath(), is_(equal_to(test_7zip_file)), 'InflateFileDone message attribute ''filepath'' is wrong')		

	def test_7z_pump_failure(self):
		test_7z_file = os.path.join(actorstestutils.current_dirpath(__file__), 'data', 'data_failure.7z')

		pump_tester = actorstestutils.TestActorBuilder().with_type(filepump.FilePumpActor)\
														.termination_type(fp_messages.InflateFileFailed)\
														.verbosity(VerbosityLevel.DEBUG)\
														.termination_moratorium(2)\
														.build()

		pump_tester.tell(fp_messages.InflateFile(test_7z_file))

		assert_that(pump_tester.inspector().num_instances(fp_messages.InflateFileFailed), is_(equal_to(1)), 'Total amount of InflatedFileFailed messages received is wrong')
		assert_that(pump_tester.inspector().first(fp_messages.InflateFileFailed).filepath(), is_(not_none()), 'InflatedFileFailed ''filepath'' is empty')
		assert_that(pump_tester.inspector().first(fp_messages.InflateFileFailed).filepath(), is_(equal_to(test_7z_file)), 'InflatedFileFailed ''filepath'' is wrong')			

	def test_unrecognized_pump_format(self):
		test_zip_file = os.path.join(actorstestutils.current_dirpath(__file__), 'data', 'unrecognized_format.kk')

		pump_tester = actorstestutils.TestActorBuilder().with_type(filepump.FilePumpActor)\
														.termination_type(fp_messages.InflateFileFailed)\
														.verbosity(VerbosityLevel.DEBUG)\
														.termination_moratorium(2)\
														.build()

		pump_tester.tell(fp_messages.InflateFile(test_zip_file))

		assert_that(pump_tester.inspector().num_instances(fp_messages.InflateFileFailed), is_(equal_to(1)), 'Total amount of InflatedFileFailed messages received is wrong')
		assert_that(pump_tester.inspector().first(fp_messages.InflateFileFailed).filepath(), is_(not_none()), 'InflatedFileFailed ''filepath'' is empty')
		assert_that(pump_tester.inspector().first(fp_messages.InflateFileFailed).filepath(), is_(equal_to(test_zip_file)), 'InflatedFileFailed ''filepath'' is wrong')

if __name__ == '__main__':
    unittest.main()		


import hashlib
import sys

class MessageTrackingGenerator:

	@staticmethod
	def generate(message):
		return MessageTrackingGenerator.__create_id(message.sourcepath())

	@staticmethod
	def generate_parent(message):
		return MessageTrackingGenerator.__create_id(message.parent_sourcepath())

	@staticmethod
	def __create_id(from_value):
		return hashlib.sha224(from_value.encode(sys.getdefaultencoding(),'replace')).hexdigest()


import unittest
import os

from longtang.actors import actorstestutils
from longtang.actors.outbound.amazon import albumcoverart, messages
from longtang.common import domain as common_domain
from hamcrest import *


class TestAlbumCoverArtActor(unittest.TestCase):

	def setUp(self):

		global tmpdir

		tmpdir=actorstestutils.create_tmp_dir(suffix='_albumart')

	def tearDown(self):
		actorstestutils.remove_dir_content(tmpdir)

	def test_single_file_match(self):

		coverart_tester = actorstestutils.TestActorBuilder().with_type(albumcoverart.AlbumCoverArtActor) \
															.verbosity(None) \
															.build()

		coverart_tester.tell(messages.LookupCoverArt('Carcass', 'Swansong', tmpdir))

		assert_that(coverart_tester.inspector().num_instances(messages.CoverArtFound), is_(equal_to(1)), 'Total amount of CoverArtFound messages received is wrong')

		message = coverart_tester.inspector().first(messages.CoverArtFound)
		assert_that(os.path.isfile(message.cover_path()), is_(equal_to(True)),'Cover file does not exist')		


#http://docs.python.org/2/library/subprocess.html#frequently-used-arguments
#7z x pump_3CL0Nm.7z -o./test

import base
import subprocess

class SevenzPumpingStrategy(base.PumpingStrategy):
	def open(self, filepath):
		try:
			return SevenzFileReference(filepath)
		except Exception, e:
			raise base.FileCouldNotBeOpenException(filepath, str(e))

	def inflate_at(self, file_ref, target_path):
		try:
			captured_output = subprocess.check_output(["7z", "x", file_ref.ref(), "-o" + target_path])
			return base.BaseInflateDefails(target_path)
		except subprocess.CalledProcessError, e:
			raise base.InflateException(target_path, str(e))

class SevenzFileReference(base.InflatedFileReference):
	def __init__(self, sevenz_file):
		self.__7z_file=sevenz_file

	def close(self):
		pass

	def ref(self):
		return self.__7z_file


from longtang.common.mediatracking import events as mt_events

class MetadataAvailable(mt_events.MediaFileTrackingEvent):
	def __init__(self):
		mt_events.MediaFileTrackingEvent.__init__(self, "METADATA_AVAILABLE", "Metadata is fully available")

class MetadataNotAvailable(mt_events.MediaFileTrackingEvent):
	def __init__(self):
		mt_events.MediaFileTrackingEvent.__init__(self, "METADATA_NOT_AVAILABLE_FAILURE", "Metadata could not be completed")

class MetadataEvaluationFailed(mt_events.MediaFileTrackingEvent):
	def __init__(self):
		mt_events.MediaFileTrackingEvent.__init__(self, "METADATA_EVALUATION_FAILURE", "Metadata could not be evaluated from file")		

class MetadataInspectionStarted(mt_events.MediaFileTrackingEvent):
	def __init__(self):
		mt_events.MediaFileTrackingEvent.__init__(self, "METADATA_INSPECTION_STARTED", "Metadata inspection from file started")

class FileMetadataIncomplete(mt_events.MediaFileTrackingEvent):
	def __init__(self):
		mt_events.MediaFileTrackingEvent.__init__(self, "FILE_METADATA_INCOMPLETE", "Metadata is incomplete. Trying MusicBrainz")

class MusicbrainzServiceFailed(mt_events.MediaFileTrackingEvent):
	def __init__(self):
		mt_events.MediaFileTrackingEvent.__init__(self, "MUSICBRAINZ_SERVICE_FAILURE", "Musicbrainz service is not responding")	



from longtang.common.mediatracking import mediatracking

class ActivitySummary():

	def __init__(self, tracking_info, compressed_files_info):
		self.__tracking_info=tracking_info
		self.__compressed_files_info=compressed_files_info

	@staticmethod
	def create(tracking_info, compressed_files_info):
		return ActivitySummary(tracking_info, compressed_files_info)

	def compressed(self):
		return ActivityEntriesClassifier(self.__compressed_files_info)

	def audio(self):
		return ActivityEntriesClassifier(self.__tracking_info)

class ActivityEntriesBrowser():

	def __init__(self, entries):
		self.__entries=entries

	def total(self):
		return len(self.__entries)

	def all(self):
		return self.__entries

	def find(self, tracking_id):
		#I think I need to review go to perform this instead of linear cost
		result = [entry for entry in self.__entries if entry[0] is tracking_id]

		if len(result) > 0:
			return result[0]
		else:
			None

class ActivityEntriesClassifier():

	def __init__(self, tracking_info):
		self.__tracking_info=tracking_info

	def total(self):
		return len(self.__tracking_info)

	def __is_failure(self, event):
		return 'FAILURE' in event.type()

	def __is_success(self, event):
		return not self.__is_failure(event)

	def __mediafile_failed(self, mediafile_tracking):
		for event in mediafile_tracking.tracking_events:
			if self.__is_failure(event):
				return True

		return False

	def __is_compressed_audio(self, mediafile_tracking):
		return isinstance(mediafile_tracking.mediafile, mediatracking.CompressedAudioFileDescriptor)

	def successes(self, include_compressed=True):

		successful_file_criteria=lambda mediatracking_info: not self.__mediafile_failed(mediatracking_info[1])
		non_compressed_successful_files_criteria=lambda mediatracking_info: successful_file_criteria(mediatracking_info) and not self.__is_compressed_audio(mediatracking_info[1])

		if include_compressed:
			entries=filter(successful_file_criteria,self.__tracking_info.items())
		else:
			entries=filter(non_compressed_successful_files_criteria,self.__tracking_info.items())

		return ActivityEntriesBrowser(entries)

	def failures(self, include_compressed=True):
		failed_file_criteria=lambda mediatracking_info: self.__mediafile_failed(mediatracking_info[1])
		non_compressed_successful_files_criteria=lambda mediatracking_info: failed_file_criteria(mediatracking_info) and not self.__is_compressed_audio(mediatracking_info[1])

		if include_compressed:
			entries=filter(failed_file_criteria,self.__tracking_info.items())
		else:
			entries=filter(non_compressed_successful_files_criteria,self.__tracking_info.items())

		return ActivityEntriesBrowser(entries)

	def all(self):
		return self.__tracking_info.items()


import base
import zipfile


class ZipPumpingStrategy(base.PumpingStrategy):
	def open(self, filepath):
		try:
			return ZipFileReference(zipfile.ZipFile(filepath, 'r'))
		except Exception, e:
			raise base.FileCouldNotBeInflatedException(filepath, str(e))

	def inflate_at(self, file_ref, target_path):
		try:
			file_ref.ref().extractall(target_path)
			return base.BaseInflateDefails(target_path)
		except Exception, e:
			raise base.InflateException(target_path, str(e))

class ZipFileReference(base.InflatedFileReference):
	def __init__(self, zipfile):
		self.__zipfile=zipfile

	def close(self):
		self.__zipfile.close()

	def ref(self):
		return self.__zipfile



#https://github.com/hamcrest/PyHamcrest
#http://packages.python.org/PyHamcrest/

from longtang.common.mediatracking import mediatracking, events as media_events
import unittest

from hamcrest import *

class MediaTrackingTest(unittest.TestCase):

	def test_creation(self):

		full_filepath = '/dummy_path.mp3'
		descriptor = mediatracking.AudioFileDescriptorBuilder().sourcepath(full_filepath).build()
		file_tracking = mediatracking.MediaFileTracking.create_from(descriptor)

		assert_that(file_tracking.mediafile.sourcepath, is_(not_none()), 'Media file path is empty')
		assert_that(file_tracking.mediafile.sourcepath, is_(equal_to(full_filepath)), 'Media file path value is wrong')
		assert_that(file_tracking.tracking_events, is_(not_none()), 'Tracking events has not being initialized')
		assert_that(file_tracking.tracking_events.is_empty(), is_(True), 'Tracking events is not empty')

	def test_add_tracking_event(self):

		full_filepath = '/dummy_path.mp3'
		descriptor = mediatracking.AudioFileDescriptorBuilder().sourcepath(full_filepath).build()
		file_tracking = mediatracking.MediaFileTracking.create_from(descriptor)

		file_tracking.track_activity(media_events.MediaFileTrackingEvent('my-unique-type', 'short description', {'value-of-interest1':1000, 'value-of-interes2':'testing'}))		

		assert_that(file_tracking.tracking_events.is_empty(), is_(False), 'Tracking events is empty')
		assert_that(file_tracking.tracking_events.all(), is_(not_none()), 'All tracking event list is empty')
		assert_that(file_tracking.tracking_events.all().size(), is_(equal_to(1)), 'All tracking event list has not the right size')
		assert_that(file_tracking.tracking_events.of_type('my-unique-type').is_empty(), is_(False), 'Tracking events of type \'my-unique-type\' is empty')
		assert_that(file_tracking.tracking_events.of_type('my-unique-type').get(0), is_(not_none()), 'First tracking event could not be found')
		assert_that(file_tracking.tracking_events.of_type('my-unique-type').all(), is_(not_none()), 'First tracking event could not be found')
		assert_that(file_tracking.tracking_events.of_type('my-unique-type').all().size(), is_(equal_to(1)), 'Tracking event list of type \'my-unique-type\' has not the right size')

	def test_add_altering_tracking_event(self):

		class DestinationUpdateTrackingEvent(media_events.MediaFileTrackingEvent):
			def __init__(self, entry_type, description, destination):
				media_events.MediaFileTrackingEvent.__init__(self, entry_type, description,{"destination": destination})

			def process(self, file_descriptor):
				file_descriptor.destinationpath=self.context_information()["destination"]
				return file_descriptor

		full_filepath = '/dummy_path.mp3'
		descriptor = mediatracking.AudioFileDescriptorBuilder().sourcepath(full_filepath).build()
		file_tracking = mediatracking.MediaFileTracking.create_from(descriptor)

		file_tracking.track_activity(media_events.MediaFileTrackingEvent('my-unique-type', 'short description', {'value-of-interest1':1000, 'value-of-interes2':'testing'}))

		assert_that(file_tracking.mediafile.destinationpath, is_(none()), "Destination path is not empty as expected")

		file_tracking.track_activity(DestinationUpdateTrackingEvent('destination-update', 'This is an altering event', '/target/path/dummy.mp3'))

		assert_that(file_tracking.mediafile.destinationpath, is_(equal_to('/target/path/dummy.mp3')), "Destination path has not been updated")
		assert_that(file_tracking.tracking_events.all().size(), is_(equal_to(2)), 'All tracking event list has not the right size')
		assert_that(file_tracking.tracking_events.of_type('destination-update').is_empty(), is_(False), 'Tracking events of type \'my-unique-type\' is empty')
		assert_that(file_tracking.tracking_events.of_type('destination-update').all().size(), is_(equal_to(1)), 'Tracking event list of type \'my-unique-type\' has not the right size')

	def test_add_compressed_file_tracking_event(self):
		full_filepath = '/dummy_path.rar'

		descriptor = mediatracking.CompressedFileDescriptorBuilder().sourcepath(full_filepath).build()

		file_tracking = mediatracking.MediaFileTracking.create_from(descriptor)

		assert_that(file_tracking.mediafile.sourcepath, is_(not_none()), 'Media file path is empty')
		assert_that(file_tracking.mediafile.sourcepath, is_(equal_to(full_filepath)), 'Media file path value is wrong')
		assert_that(file_tracking.tracking_events, is_(not_none()), 'Tracking events has not being initialized')
		assert_that(file_tracking.tracking_events.is_empty(), is_(True), 'Tracking events is not empty')
		assert_that(file_tracking.mediafile.audiofiles, is_(not_none()), 'Related audiofile list has not been initialized')
		assert_that(file_tracking.mediafile.inflated, is_(equal_to(True)), 'Related inflated status is not initialized')		

	def test_relate_audiofile_to_compressed_file_tracking(self):
		full_filepath = '/dummy_path.rar'

		descriptor = mediatracking.CompressedFileDescriptorBuilder().sourcepath(full_filepath).build()

		file_tracking = mediatracking.MediaFileTracking.create_from(descriptor)

		file_tracking.track_activity(media_events.CompressedAudioFileFound('tracking-id1'))
		file_tracking.track_activity(media_events.CompressedAudioFileFound('tracking-id2'))

		assert_that(file_tracking.tracking_events.of_type('COMPRESSED_AUDIOFILE_FOUND').is_empty(), is_(False), 'Tracking events of type \'COMPRESSED_AUDIOFILE_FOUND\' is empty')		
		assert_that(file_tracking.tracking_events.of_type('COMPRESSED_AUDIOFILE_FOUND').all().size(), is_(equal_to(2)), 'Tracking event list of type \'COMPRESSED_AUDIOFILE_FOUND\' has not the right size')
		assert_that(file_tracking.mediafile.audiofiles.size(), is_(equal_to(2)), 'Related audiofile list has not been initialized')
		assert_that(file_tracking.mediafile.audiofiles.all(), has_items('tracking-id1','tracking-id2'), 'Related audiofile list has not the expected items')

	def test_altering_tracking_event_on_compressed_file(self):
		full_filepath = '/dummy_path.rar'

		descriptor = mediatracking.CompressedFileDescriptorBuilder().sourcepath(full_filepath).build()

		file_tracking = mediatracking.MediaFileTracking.create_from(descriptor)

		file_tracking.track_activity(media_events.FileCouldNotBeInflated())

		assert_that(file_tracking.mediafile.sourcepath, is_(not_none()), 'Media file path is empty')
		assert_that(file_tracking.mediafile.sourcepath, is_(equal_to(full_filepath)), 'Media file path value is wrong')
		assert_that(file_tracking.tracking_events, is_(not_none()), 'Tracking events has not being initialized')
		assert_that(file_tracking.tracking_events.is_empty(), is_(equal_to(False)), 'Tracking events is empty')
		assert_that(file_tracking.tracking_events.of_type('INFLATE_PROCESS_FAILURE').is_empty(), is_(False), 'Tracking events of type \'INFLATE_PROCEDURE_FAILED\' is empty')	
		assert_that(file_tracking.mediafile.inflated, is_(equal_to(False)), 'Related inflated status is not initialized')		

	def test_add_compressed_mediafile_tracking_event(self):
		full_filepath = '/dummy_path.mp3'
		descriptor = mediatracking.CompressedAudioFileDescriptorBuilder().sourcepath(full_filepath)\
																		 .compressed_file('parent-tracking-id').build()
		
		file_tracking = mediatracking.MediaFileTracking.create_from(descriptor)

		file_tracking.track_activity(media_events.MediaFileTrackingEvent('my-unique-type', 'short description', {'value-of-interest1':1000, 'value-of-interes2':'testing'}))		

		assert_that(file_tracking.tracking_events.is_empty(), is_(False), 'Tracking events is empty')
		assert_that(file_tracking.tracking_events.all(), is_(not_none()), 'All tracking event list is empty')
		assert_that(file_tracking.tracking_events.all().size(), is_(equal_to(1)), 'All tracking event list has not the right size')
		assert_that(file_tracking.tracking_events.of_type('my-unique-type').is_empty(), is_(False), 'Tracking events of type \'my-unique-type\' is empty')
		assert_that(file_tracking.tracking_events.of_type('my-unique-type').get(0), is_(not_none()), 'First tracking event could not be found')
		assert_that(file_tracking.tracking_events.of_type('my-unique-type').all(), is_(not_none()), 'First tracking event could not be found')
		assert_that(file_tracking.tracking_events.of_type('my-unique-type').all().size(), is_(equal_to(1)), 'Tracking event list of type \'my-unique-type\' has not the right size')
		assert_that(file_tracking.mediafile.compressed_file, is_(equal_to('parent-tracking-id')), 'Parent compressed file has not been informed')

	def test_add_altering_tracking_event_on_compressed_mediafile(self):

		class DestinationUpdateTrackingEvent(media_events.MediaFileTrackingEvent):
			def __init__(self, entry_type, description, destination):
				media_events.MediaFileTrackingEvent.__init__(self, entry_type, description,{"destination": destination})

			def process(self, file_descriptor):
				file_descriptor.destinationpath=self.context_information()["destination"]
				return file_descriptor

		full_filepath = '/dummy_path.mp3'
		descriptor = mediatracking.CompressedAudioFileDescriptorBuilder().sourcepath(full_filepath)\
																		 .compressed_file('parent-tracking-id').build()
																		 
		file_tracking = mediatracking.MediaFileTracking.create_from(descriptor)

		file_tracking.track_activity(media_events.MediaFileTrackingEvent('my-unique-type', 'short description', {'value-of-interest1':1000, 'value-of-interes2':'testing'}))

		assert_that(file_tracking.mediafile.destinationpath, is_(none()), "Destination path is not empty as expected")

		file_tracking.track_activity(DestinationUpdateTrackingEvent('destination-update', 'This is an altering event', '/target/path/dummy.mp3'))

		assert_that(file_tracking.mediafile.destinationpath, is_(equal_to('/target/path/dummy.mp3')), "Destination path has not been updated")
		assert_that(file_tracking.tracking_events.all().size(), is_(equal_to(2)), 'All tracking event list has not the right size')
		assert_that(file_tracking.tracking_events.of_type('destination-update').is_empty(), is_(False), 'Tracking events of type \'my-unique-type\' is empty')
		assert_that(file_tracking.tracking_events.of_type('destination-update').all().size(), is_(equal_to(1)), 'Tracking event list of type \'my-unique-type\' has not the right size')


import os

class PumpingStrategy:
	def open(self, filepath):
		pass

	def inflate_at(self, file_ref, target_path):
		pass

class InflatedFileReference:
	def close(self):
		pass

	def ref(self):
		pass

class BaseInflateDefails:
	def __init__(self, inflated_at):
		self.__inflated_at=inflated_at

	def entries(self):
		for root, dirnames, filenames in os.walk(self.__inflated_at):
			for filename in filenames:
				yield os.path.join(root, filename)

class FileCouldNotBeOpenException(Exception):
	def __init__(self, filename, msg=None):
		self.__filename = filename
		self.__msg = msg		

	def __str__(self):
		return repr("File '{0}' could not be open. Reason: {1}".format(self.__filename, self.__msg if self.__msg is not None else ''))

	def filename(self):
		return self.__filename

class InflateException(Exception):
	def __init__(self, filename, msg=None):
		self.__filename = filename
		self.__msg = msg		

	def __str__(self):
		return repr("File '{0}' could not be inflated. Reason: {1}".format(self.__filename, self.__msg if self.__msg is not None else ''))

	def filename(self):
		return self.__filename		


from longtang.actors import messages

class InflateFile(messages.ActorMessage):

	def __init__(self, filepath):
		messages.ActorMessage.__init__(self)
		self.__filepath=filepath

	def filepath(self):
		return self.__filepath

class InflatedFileAvailable(messages.ActorMessage):

	def __init__(self, filepath, parent):
		messages.ActorMessage.__init__(self)
		self.__filepath=filepath
		self.__parent=parent

	def filepath(self):
		return self.__filepath		

	def parent_filepath(self):
		return self.__parent

class InflateFileDone(messages.ActorMessage):

	def __init__(self, filepath):
		messages.ActorMessage.__init__(self)
		self.__filepath=filepath

	def filepath(self):
		return self.__filepath

class InflateFileFailed(messages.ActorMessage):

	def __init__(self, filepath):
		messages.ActorMessage.__init__(self)
		self.__filepath=filepath

	def filepath(self):
		return self.__filepath


class FilePackagingConfiguration():
	
	def __init__(self):
		self.__offline_mode=None
		self.__is_enable=None

	@property
	def cover_art_enabled(self):
		return self.__is_enable

	@cover_art_enabled.setter
	def cover_art_enabled(self, value):
		self.__is_enable = value

	@property
	def offline_mode(self):
		return self.__offline_mode

	@offline_mode.setter
	def offline_mode(self, value):
		self.__offline_mode = value

class FilePackagingConfigurationBuilder():

	def __init__(self):
		self.__offline_mode=False
		self.__enabled=True

	def offline_mode(self, value):
		self.__offline_mode=value
		return self

	def cover_art_enabled(self, value):
		self.__enabled=value
		return self

	def build(self):
		config = FilePackagingConfiguration()

		config.offline_mode=self.__offline_mode
		config.cover_art_enabled=self.__enabled

		return config


#https://github.com/hamcrest/PyHamcrest
#http://packages.python.org/PyHamcrest/

import gevent
import unittest
import os

from hamcrest import *
from longtang.actors.tracking import tracking, messages
from longtang.system import system
from longtang.actors import actorstestutils
from longtang.common import interactions
from longtang.common.mediatracking import mediatracking, events as mt_events

class TestMediaTrackingActor(unittest.TestCase):

	def test_creation(self):
		actor_system = system.ActorSystem()
		tracking_actor = actor_system.from_type(tracking.TrackingActor,'tracking-actor')

		try:
			assert_that(actor_system.find_by_id('tracking-actor'), is_not(None), 'Tracking actor does not exist within system')
		except exceptions.ActorNotFound as e:
			self.fail(str(e))
						
		actor_system.shutdown()

	def test_add_single_tracking(self):
		actor_system = system.ActorSystem()
		tracking_actor = actor_system.from_type(tracking.TrackingActor,'tracking-actor')

		response = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.CreateTrackingEntry('/dummy.mp3'))

		assert_that(response, is_(not_none()),'Tracking identifier message is empty')
		assert_that(response, is_(instance_of(messages.TrackingEntryCreated)),'Tracking response message is not of the right type')
		assert_that(response.tracking_id(), is_(not_none()),'Tracking identifier is missing')

		actor_system.shutdown()

	def test_retrieve_single_tracking(self):
		actor_system = system.ActorSystem()
		tracking_actor = actor_system.from_type(tracking.TrackingActor,'tracking-actor')		

		tracking_id = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.CreateTrackingEntry('/dummy-2.mp3')).tracking_id()

		response = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.LookupTrackingEntry(tracking_id))

		assert_that(response, is_(not_none()),'Tracking information message is empty')
		assert_that(response, is_(instance_of(messages.TrackingEntryFound)),'Tracking information message is not of the right type')
		assert_that(response.tracking_info().mediafile.sourcepath, is_(equal_to('/dummy-2.mp3')),'Tracking identifier is missing')

		actor_system.shutdown()

	def test_retrieve_single_tracking_notfound(self):
		actor_system = system.ActorSystem()
		tracking_actor = actor_system.from_type(tracking.TrackingActor,'tracking-actor')		

		tracking_id = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.CreateTrackingEntry('/dummy-2.mp3')).tracking_id()

		response = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.LookupTrackingEntry('nonexistingid'))

		assert_that(response, is_(not_none()),'Tracking information message is empty')
		assert_that(response, is_(instance_of(messages.TrackingEntryNotFound)),'Tracking information message is not of the right type')
		assert_that(response.tracking_id(), is_(equal_to('nonexistingid')),'Tracking identifier is missing')		

		actor_system.shutdown()

	def test_notify_non_altering_tracking_event(self):
		actor_system = system.ActorSystem()
		tracking_actor = actor_system.from_type(tracking.TrackingActor,'tracking-actor')		

		tracking_id = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.CreateTrackingEntry('/dummy-non-tracking.mp3')).tracking_id()

		event = mt_events.MediaFileTrackingEvent('NONALTERING', 'This is a non-altering event')

		response = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.RegisterTrackingEvent(tracking_id,event))

		assert_that(response, is_(not_none()),'Tracking information message is empty')
		assert_that(response, is_(instance_of(messages.TrackingEventSuccessfullyRegistered)),'Tracking event response is not of the right type')

		response = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.LookupTrackingEntry(tracking_id))

		assert_that(response.tracking_info().tracking_events.of_type('NONALTERING').is_empty(), is_(False),'Tracking events of type NONALTERING are empty')
		assert_that(response.tracking_info().tracking_events.of_type('NONALTERING').get(0), is_(not_none()),'Tracked events does not exist')

		actor_system.shutdown()

	def test_notify_altering_tracking_event(self):
		actor_system = system.ActorSystem()
		tracking_actor = actor_system.from_type(tracking.TrackingActor,'tracking-actor')		

		tracking_id = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.CreateTrackingEntry('/dummy-non-tracking.mp3')).tracking_id()

		event = DestinationUpdateTrackingEvent('DESTINATION', 'This is an altering event', '/target/path/dummy.mp3')

		response = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.RegisterTrackingEvent(tracking_id,event))

		assert_that(response, is_(not_none()),'Tracking information message is empty')
		assert_that(response, is_(instance_of(messages.TrackingEventSuccessfullyRegistered)),'Tracking event response is not of the right type')

		response = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.LookupTrackingEntry(tracking_id))

		assert_that(response.tracking_info().tracking_events.of_type('DESTINATION').is_empty(), is_(False),'Tracking events of type NONALTERING are empty')
		assert_that(response.tracking_info().tracking_events.of_type('DESTINATION').get(0), is_(not_none()),'Tracked events does not exist')
		assert_that(response.tracking_info().mediafile.destinationpath, is_(equal_to('/target/path/dummy.mp3')),'Mediafile information has not been updated')

		actor_system.shutdown()
	
	def test_add_compressed_file_tracking(self):
		actor_system = system.ActorSystem()
		tracking_actor = actor_system.from_type(tracking.TrackingActor,'tracking-actor')

		response = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.CreateCompressedFileTrackingEntry('/dummy.zip'))

		assert_that(response, is_(not_none()),'Tracking identifier message is empty')
		assert_that(response, is_(instance_of(messages.CompressedFileTrackingEntryCreated)),'Tracking response message is not of the right type')
		assert_that(response.tracking_id(), is_(not_none()),'Tracking identifier is missing')
		assert_that(response.tracking_id(), is_(has_length(greater_than(0))),'Tracking identifier is empty')

		actor_system.shutdown()

	def test_add_compressed_audio_file_tracking(self):
		actor_system = system.ActorSystem()
		tracking_actor = actor_system.from_type(tracking.TrackingActor,'tracking-actor')

		response = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.CreateCompressedAudioFileTrackingEntry('mymusic.mp3', '/dummy.zip'))

		assert_that(response, is_(not_none()),'Tracking identifier message is empty')
		assert_that(response, is_(instance_of(messages.CompressedAudioFileTrackingEntryCreated)),'Tracking response message is not of the right type')
		assert_that(response.tracking_id(), is_(not_none()),'Tracking identifier is missing')
		assert_that(response.tracking_id(), is_(has_length(greater_than(0))),'Tracking identifier is empty')
		assert_that(response.parent_tracking_id(), is_(not_none()),'Tracking identifier is missing')
		assert_that(response.parent_tracking_id(), is_(has_length(greater_than(0))),'Tracking identifier is empty')

		actor_system.shutdown()

	def test_notity_compressed_file_tracking_event(self):
		actor_system = system.ActorSystem()
		tracking_actor = actor_system.from_type(tracking.TrackingActor,'tracking-actor')

		tracking_id = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.CreateCompressedFileTrackingEntry('/dummy.zip')).tracking_id()

		event = mt_events.MediaFileTrackingEvent('NONALTERING', 'This is a non-altering event')

		response = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.RegisterCompressedFileTrackingEvent(tracking_id,event))

		assert_that(response, is_(not_none()),'Tracking information message is empty')
		assert_that(response, is_(instance_of(messages.TrackingEventSuccessfullyRegistered)),'Tracking event response is not of the right type')

		response = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.LookupTrackingEntry(tracking_id))

		assert_that(response, is_(instance_of(messages.TrackingEntryFound)), 'Compressed file tracking information was not found')
		assert_that(response.tracking_info().tracking_events.of_type('NONALTERING').is_empty(), is_(False),'Tracking events of type NONALTERING are empty')
		assert_that(response.tracking_info().tracking_events.of_type('NONALTERING').get(0), is_(not_none()),'Tracked events does not exist')

		actor_system.shutdown()

	def test_check_audiofiles_list_from_compressed_file(self):
		actor_system = system.ActorSystem()
		tracking_actor = actor_system.from_type(tracking.TrackingActor,'tracking-actor')

		audio1_response = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.CreateCompressedAudioFileTrackingEntry('mymusic1.mp3', '/dummy.zip'))
		audio2_response = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.CreateCompressedAudioFileTrackingEntry('mymusic2.mp3', '/dummy.zip'))

		assert_that(audio1_response.parent_tracking_id(), is_(equal_to(audio2_response.parent_tracking_id())), 'Parent compressed file tracking id does not match')

		response = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.LookupTrackingEntry(audio2_response.parent_tracking_id()))

		assert_that(response.tracking_info().mediafile.audiofiles.size(), is_(equal_to(2)),'Audio files are not registered at compressed file tracking')
		assert_that(response.tracking_info().mediafile.audiofiles.all(), has_items(audio1_response.tracking_id(), audio2_response.tracking_id()), 'Compressed audio files are not accurately related to the compressed file')

	def test_notify_compressed_audio_file_non_altering_tracking_event(self):
		actor_system = system.ActorSystem()
		tracking_actor = actor_system.from_type(tracking.TrackingActor,'tracking-actor')		

		tracking_id = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.CreateCompressedAudioFileTrackingEntry('mymusic.mp3', '/dummy.zip')).tracking_id()

		event = mt_events.MediaFileTrackingEvent('NONALTERING', 'This is a non-altering event')

		response = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.RegisterTrackingEvent(tracking_id,event))

		assert_that(response, is_(not_none()),'Tracking information message is empty')
		assert_that(response, is_(instance_of(messages.TrackingEventSuccessfullyRegistered)),'Tracking event response is not of the right type')

		response = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.LookupTrackingEntry(tracking_id))

		assert_that(response.tracking_info().tracking_events.of_type('NONALTERING').is_empty(), is_(False),'Tracking events of type NONALTERING are empty')
		assert_that(response.tracking_info().tracking_events.of_type('NONALTERING').get(0), is_(not_none()),'Tracked events does not exist')

		actor_system.shutdown()

	def test_notify_compressed_audio_file_altering_tracking_event(self):
		actor_system = system.ActorSystem()
		tracking_actor = actor_system.from_type(tracking.TrackingActor,'tracking-actor')		

		tracking_id = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.CreateCompressedAudioFileTrackingEntry('mymusic.mp3', '/dummy.zip')).tracking_id()

		event = DestinationUpdateTrackingEvent('DESTINATION', 'This is an altering event', '/target/path/dummy.mp3')

		response = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.RegisterTrackingEvent(tracking_id,event))

		assert_that(response, is_(not_none()),'Tracking information message is empty')
		assert_that(response, is_(instance_of(messages.TrackingEventSuccessfullyRegistered)),'Tracking event response is not of the right type')

		response = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.LookupTrackingEntry(tracking_id))

		assert_that(response.tracking_info().tracking_events.of_type('DESTINATION').is_empty(), is_(False),'Tracking events of type NONALTERING are empty')
		assert_that(response.tracking_info().tracking_events.of_type('DESTINATION').get(0), is_(not_none()),'Tracked events does not exist')
		assert_that(response.tracking_info().mediafile.destinationpath, is_(equal_to('/target/path/dummy.mp3')),'Mediafile information has not been updated')

	def test_retrieve_summary_without_compressed_files(self):
		actor_system = system.ActorSystem()
		tracking_actor = actor_system.from_type(tracking.TrackingActor,'tracking-actor')		

		audio1_tracking_id = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.CreateTrackingEntry('/dummy1.mp3')).tracking_id()
		audio2_tracking_id = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.CreateTrackingEntry('/dummy2.mp3')).tracking_id()
		audio3_tracking_id = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.CreateTrackingEntry('/dummy3.mp3')).tracking_id()

		summary = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.GenerateSummary()).summary()

		assert_that(summary.audio().total(), is_(equal_to(3)), 'Number of files in the summary is wrong')
		assert_that(summary.audio().successes().total(), is_(equal_to(3)), 'Number of summary successes is wrong')
		assert_that(summary.audio().failures().total(), is_(equal_to(0)), 'Number of failures successes is wrong')

		total = summary.audio().total()

		assert_that(summary.audio().successes().find(audio1_tracking_id), is_(not_none()), 'Audio1 tracking id could not be found')
		assert_that(summary.audio().successes().find(audio2_tracking_id), is_(not_none()), 'Audio2 tracking id could not be found')
		assert_that(summary.audio().successes().find(audio3_tracking_id), is_(not_none()), 'Audio3 tracking id could not be found')

	def test_retrieve_summary_without_compressed_files_and_single_failure(self):
		actor_system = system.ActorSystem()
		tracking_actor = actor_system.from_type(tracking.TrackingActor,'tracking-actor')		

		audio1_tracking_id = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.CreateTrackingEntry('/dummy1.mp3')).tracking_id()
		audio2_tracking_id = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.CreateTrackingEntry('/dummy2.mp3')).tracking_id()
		audio3_tracking_id = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.CreateTrackingEntry('/dummy3.mp3')).tracking_id()

		event = mt_events.MediaFileTrackingEvent('THIS_IS_A_FAILURE', 'This is a non-altering event')

		response = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.RegisterTrackingEvent(audio1_tracking_id,event))

		summary = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.GenerateSummary()).summary()

		assert_that(summary.audio().total(), is_(equal_to(3)), 'Number of files in the summary is wrong')
		assert_that(summary.audio().successes().total(), is_(equal_to(2)), 'Number of summary successes is wrong')
		assert_that(summary.audio().failures().total(), is_(equal_to(1)), 'Number of failures successes is wrong')

		assert_that(summary.audio().successes().find(audio1_tracking_id), is_(none()), 'Audio1 tracking id was found within success!!')
		assert_that(summary.audio().successes().find(audio2_tracking_id), is_(not_none()), 'Audio2 tracking id could not be found within successes')
		assert_that(summary.audio().successes().find(audio3_tracking_id), is_(not_none()), 'Audio3 tracking id could not be found within successes')

		assert_that(summary.audio().failures().find(audio1_tracking_id), is_(not_none()), 'Audio1 tracking id could not be found within failures')
		assert_that(summary.audio().failures().find(audio2_tracking_id), is_(none()), 'Audio2 tracking id was found within failures!!')
		assert_that(summary.audio().failures().find(audio3_tracking_id), is_(none()), 'Audio3 tracking id was found within failures!!')		

	def test_retrieve_summary_with_compressed_files(self):
		actor_system = system.ActorSystem()
		tracking_actor = actor_system.from_type(tracking.TrackingActor,'tracking-actor')		

		audio1_tracking_id = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.CreateTrackingEntry('/dummy1.mp3')).tracking_id()
		audio2_tracking_id = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.CreateTrackingEntry('/dummy2.mp3')).tracking_id()
		audio3_tracking_id = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.CreateTrackingEntry('/dummy3.mp3')).tracking_id()
		compressed1_audio1_tracking_id = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.CreateCompressedAudioFileTrackingEntry('mymusic1.mp3', '/dummy1.zip')).tracking_id()
		compressed1_audio2_tracking_id = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.CreateCompressedAudioFileTrackingEntry('mymusic2.mp3', '/dummy1.zip')).tracking_id()		
		compressed2_audio1_tracking_id = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.CreateCompressedAudioFileTrackingEntry('myothermusic1.mp3', '/dummy2.zip')).tracking_id()		
		compressed2_audio2_tracking_id = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.CreateCompressedAudioFileTrackingEntry('myothermusic2.mp3', '/dummy2.zip')).tracking_id()

		summary = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.GenerateSummary()).summary()

		assert_that(summary.audio().total(), is_(equal_to(7)), 'Number of files in the summary is wrong')
		assert_that(summary.audio().successes().total(), is_(equal_to(7)), 'Number of summary successes is wrong')
		assert_that(summary.audio().successes(False).total(), is_(equal_to(3)), 'Number of summary successes without compressed audio files is wrong')
		assert_that(summary.audio().failures().total(), is_(equal_to(0)), 'Number of failures successes is wrong')
		assert_that(summary.audio().failures(False).total(), is_(equal_to(0)), 'Number of failures successes without compressed audio files is wrong')
		assert_that(summary.compressed().successes().total(), is_(equal_to(2)), 'Number of success uncompressed files is wrong')
		assert_that(summary.compressed().failures().total(), is_(equal_to(0)), 'Number of success uncompressed files is wrong')

		for tracking_id, mediatracking in summary.compressed().all():
			assert_that(mediatracking.mediafile.inflated, is_(equal_to(True)), 'All compressed files are not marked as inflated')

	def test_retrieve_summary_with_compressed_files_and_single_compressed_file_failure(self):
		actor_system = system.ActorSystem()
		tracking_actor = actor_system.from_type(tracking.TrackingActor,'tracking-actor')		

		audio1_tracking_id = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.CreateTrackingEntry('/dummy1.mp3')).tracking_id()
		audio2_tracking_id = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.CreateTrackingEntry('/dummy2.mp3')).tracking_id()
		audio3_tracking_id = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.CreateTrackingEntry('/dummy3.mp3')).tracking_id()

		compressedfile1_tracking_id = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.CreateCompressedFileTrackingEntry('/dummy1.zip')).tracking_id()
		
		compressed2_audio1_tracking_id = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.CreateCompressedAudioFileTrackingEntry('myothermusic1.mp3', '/dummy2.zip')).tracking_id()
		compressed2_audio2_tracking_id = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.CreateCompressedAudioFileTrackingEntry('myothermusic2.mp3', '/dummy2.zip')).tracking_id()

		response = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.RegisterCompressedFileTrackingEvent(compressedfile1_tracking_id,mt_events.FileCouldNotBeInflated()))

		summary = interactions.InteractionPatterns.ask(actor_system,tracking_actor, messages.GenerateSummary()).summary()

		assert_that(summary.audio().total(), is_(equal_to(5)), 'Number of files in the summary is wrong')
		assert_that(summary.audio().successes().total(), is_(equal_to(5)), 'Number of summary successes is wrong')
		assert_that(summary.audio().successes(False).total(), is_(equal_to(3)), 'Number of summary successes without compressed audio files is wrong')
		assert_that(summary.audio().failures().total(), is_(equal_to(0)), 'Number of failures successes is wrong')
		assert_that(summary.audio().failures(False).total(), is_(equal_to(0)), 'Number of failures successes without compressed audio files is wrong')
		assert_that(summary.compressed().successes().total(), is_(equal_to(1)), 'Number of success uncompressed files is wrong')
		assert_that(summary.compressed().failures().total(), is_(equal_to(1)), 'Number of success uncompressed files is wrong')

		assert_that(summary.compressed().failures().find(compressedfile1_tracking_id)[1].mediafile.inflated, is_(equal_to(False)), 'Failed compressed file is still marked as inflated')

class DestinationUpdateTrackingEvent(mt_events.MediaFileTrackingEvent):
	def __init__(self, entry_type, description, destination):
		mt_events.MediaFileTrackingEvent.__init__(self, entry_type, description,{"destination": destination})

	def process(self, file_descriptor):
		file_descriptor.destinationpath=self.context_information()["destination"]
		return file_descriptor


#http://docs.python.org/2/howto/unicode

# -*- coding: utf-8 -*-

import os
import messages
import types
import re

from longtang.actors import actors
from longtang.actors.inbound.inflate import filepump, messages as fp_messages

class FilePoller(actors.Actor):

	def __init__(self, system_ref, unique_id):
		actors.Actor.__init__(self, system_ref, unique_id)

		self.__inflate_requests = {}
		self.__caller = []

		self.__filter_criteria = lambda types: lambda y: re.search(types, y) is not None
		self.__compressed_file_criteria = self.__filter_criteria('(.zip|.rar|.7z)$')
		self.__audio_file_criteria = self.__filter_criteria('.mp3$')

	def pre_setup(self):
		
		self.logger().info('Initializing actors...')

		self.__filepump_ref = self.context().from_type(filepump.FilePumpActor,'filepump-actor')

	def receive(self, message):

		if isinstance(message, messages.StartPolling):

			self.__caller.append(self.sender())
			source = message.source()

			if type(source) is not types.UnicodeType:
				source = unicode(message.source(), encoding='utf-8', errors='replace')

			self.logger().info(u'Polling request recieved. Polling source: {0}'.format(source))

			try:
				for root, dirnames, filenames in os.walk(source):

	  				for filename in filter(self.__audio_file_criteria, filenames):
	  					full_path = os.path.join(root, filename)
						self.logger().info(u'New file found: {0}'.format(full_path))
						self.sender().tell(messages.AudioFileFound(full_path), self.myself())

					for filename in filter(self.__compressed_file_criteria, filenames):
						full_path = os.path.join(root, filename)
						self.logger().info(u'New compressed file found: {0}. Requesting to be inflate'.format(full_path))

						self.__inflate_requests[full_path]=None

						self.context().get_by_id('filepump-actor').tell(fp_messages.InflateFile(full_path), self.myself())

				self.__notify_polling_done_if_possible()

			except Exception as e:
				self.logger().warn(u'Polling process interrupted. Reason: {0}'.format(str(e)))
				self.__caller[0].tell(messages.FilePollingDone(), self.myself())
		
		elif isinstance(message, fp_messages.InflateFileDone):
			self.__notify_polling_done_if_possible(message)
		elif isinstance(message, fp_messages.InflateFileFailed):
			self.__caller[0].tell(messages.CompressedFileCouldNotBeOpened(message.filepath()), self.myself())
			self.__notify_polling_done_if_possible(message)
		elif isinstance(message, fp_messages.InflatedFileAvailable):

			if self.__audio_file_criteria(message.filepath()):
				self.__caller[0].tell(messages.CompressedAudioFileFound(message.filepath(), message.parent_filepath()), self.myself())
			else:
				self.logger().debug(u'Inflated file \'{0}\' discarded'.format(message.filepath()))
		else: 
			self.notify_marooned_message(message)

	def __notify_polling_done_if_possible(self, message=None):

		if message is not None:
			self.__inflate_requests.pop(message.filepath())

		if len(self.__inflate_requests) == 0:
			self.__caller[0].tell(messages.FilePollingDone(), self.myself())


from longtang.actors.inbound.inflate.strategies import *

available_formats = {'zip' : zippumping.ZipPumpingStrategy(),\
					 'rar': rarpumping.RarPumpingStrategy(),\
					 '7z': sevenzpumping.SevenzPumpingStrategy()} 


#http://docs.python.org/2/howto/unicode

# -*- coding: utf-8 -*-

import os
import messages
import types

from pumpingclerk import PumpingClerkFactory
from longtang.actors import actors

class FilePumpActor(actors.Actor):
	def receive(self, message):
		if isinstance(message, messages.InflateFile):

			filepath = message.filepath()

			if type(filepath) is not types.UnicodeType:
				filepath = unicode(filepath, encoding='utf-8', errors='replace')

			self.logger().info(u'Inflate file request recieved. Compressed source file: {0}'.format(filepath))

			try:
				clerk = PumpingClerkFactory.from_file(filepath)

				for entry in clerk.inflate().entries():
					self.logger().debug(u'Inflated entry \'{0}\' retrieved from file {1}'.format(entry, filepath))
					self.sender().tell(messages.InflatedFileAvailable(entry, filepath), self.myself())

				self.sender().tell(messages.InflateFileDone(filepath), self.myself())

			except Exception,e:

				self.logger().error('File pumping failed. Source: {0}'.format(str(e)))
				self.sender().tell(messages.InflateFileFailed(filepath), self.myself())


		else: 
			self.notify_marooned_message(message)